require 'spec_helper'

describe TimeKeeper::CustomTime do
  subject do
    TimeKeeper::CustomTime.new("09:45 am")
  end
  describe "#initialize" do
    it "should set the time_in_str variable" do
      TimeKeeper::CustomTime.new("06:45 am").instance_variable_get('@time_in_str').should eq("06:45 am")
    end
  end

  describe "#parse" do
    it "should set the parsed time object to the appropriate time parser" do
      subject.parse.instance_variable_get("@parsed_time").class.should eq(TimeKeeper::Parser::Meridian)
    end

    it "should call parse on the time parser object" do
      time_parser_obj = TimeKeeper::Parser::Meridian.new("06:45 am")
      TimeKeeper::Parser::Base.stubs(:build).returns(time_parser_obj)
      time_parser_obj.expects(:parse)
      subject.parse
    end

    it "should return the custom time object" do
      subject.parse.should eq(subject)
    end
  end

  describe "#in_seconds" do
    context "when the time is parsed" do
      before(:each) do
        subject.instance_variable_set('@parsed_time', TimeKeeper::Parser::Meridian.new("06:45 am").parse)
      end

      it "should return the time in seconds" do
        subject.in_seconds.should eq(24300)
      end
    end

    context "when the time is not parsed" do
      before(:each) do
        subject.instance_variable_set('@parsed_time', nil)
      end

      it "should raise an exception" do
        lambda{subject.in_seconds}.should raise_exception
      end
    end
  end

  describe "#in_minutes" do
    context "when the time is parsed" do
      before(:each) do
        subject.instance_variable_set('@parsed_time', TimeKeeper::Parser::Meridian.new("06:45 am").parse)
      end

      it "should return the time in minutes" do
        subject.in_minutes.should eq(405)
      end
    end

    context "when the time is not parsed" do
      before(:each) do
        subject.instance_variable_set('@parsed_time', nil)
      end

      it "should raise an exception" do
        lambda{subject.in_minutes}.should raise_exception
      end
    end
  end
  describe "#in_hours" do
    context "when the time is parsed" do
      before(:each) do
        subject.instance_variable_set('@parsed_time', TimeKeeper::Parser::Meridian.new("06:45 am").parse)
      end

      it "should return the time in hours" do
        subject.in_hours.should eq(6.75)
      end
    end

    context "when the time is not parsed" do
      before(:each) do
        subject.instance_variable_set('@parsed_time', nil)
      end

      it "should raise an exception" do
        lambda{subject.in_minutes}.should raise_exception
      end
    end
  end

  describe "#to_s" do
    it "should return formatted hours and minutes" do
      subject.expects(:in_minutes).twice.returns(405)
      subject.to_s.should eq("06:45")
    end
  end
end