require 'spec_helper'

describe TimeKeeper::Parser::Base do

  context "when time_is_valid method is implemented" do
    before(:each) do
      TimeKeeper::Parser::Base.any_instance.stubs(:time_is_valid).returns(true)
    end

    subject do
      TimeKeeper::Parser::Base.new("11:20 am")
    end

    describe "#initialize" do
      it "should set the time in string" do
        TimeKeeper::Parser::Base.new("11:20 am").instance_variable_get('@time_str').should eq("11:20 am")
      end

      it "should raise an exception if the time is not valid" do
        lambda{TimeKeeper::Parser::Base.new("11:20 xyz")}.should raise_exception
      end
    end

    describe ".build" do
      context "when the time is in meridian format" do
        it "should return a meridian time parser object" do
          TimeKeeper::Parser::Base.build("11:20 am").class.should eq(TimeKeeper::Parser::Meridian)
        end
      end

      context "when the time is in military format" do
        it "should return a military time parser object" do
          TimeKeeper::Parser::Base.build("14:20").class.should eq(TimeKeeper::Parser::Military)
        end
      end
    end

    describe "#parse" do
      it "should raise an exception" do
        lambda{subject.parse}.should raise_exception(NotImplementedError)
      end
    end

    describe "#extract_hour" do
      it "should raise an exception" do
        lambda{subject.send(:extract_hour)}.should raise_exception(NotImplementedError)
      end
    end

    describe "#extract_min" do
      it "should assign minute to the @min variable" do
        subject.send(:extract_min, ":45")
        subject.instance_variable_get('@min').should eq(45)
      end

      context "when min is present" do
        it "should return the number of mins in integer" do
          subject.send(:extract_min, ":45").should eq(45)
        end
      end

      context "when min is nil or blank" do
        it "should return 0" do
          subject.send(:extract_min, "").should eq(0)
        end
      end
    end
  end

  context "when time_is_valid method is not implemented" do
    before(:each) do
      TimeKeeper::Parser::Base.any_instance.stubs(:valid?).returns(true)
    end

    subject do
      TimeKeeper::Parser::Base.new("06:35")
    end

    describe "#time_is_valid" do
      it "should raise an exception" do
        lambda{subject.send(:time_is_valid)}.should raise_exception(NotImplementedError)
      end
    end
  end

  describe ".meridian_format?" do
    context "when the time contains am or pm" do
      it "should return true" do
        TimeKeeper::Parser::Base.meridian_format?("6:45 Pm").should be_true
      end
    end

    context "when the time does not contain am or pm" do
      it "should return false" do
        TimeKeeper::Parser::Base.meridian_format?("18:45").should be_false
      end
    end
  end
end
