require 'spec_helper'

describe TimeKeeper::Parser::Meridian do
  subject do
    TimeKeeper::Parser::Meridian.new("7:45 pm")
  end

  describe "#time_is_valid" do
    context "when hour and mins are both valid" do
      it "should return true" do
        subject.send(:time_is_valid).should be_true
      end
    end

    context "when either hour or min is not valid" do
      before(:each) do
        subject.instance_variable_set('@time_str', "07:60 pm")
      end

      it "should return false" do
        subject.send(:time_is_valid).should be_false
      end

      it "should attach an error to the object" do
        subject.send(:time_is_valid)
        subject.errors[:base].should_not be_empty
      end
    end
  end

  describe "#extract_hour" do
    context "when the time contains 12 am" do
      it "should set hour as 0" do
        subject.send(:extract_hour, "12", "am")
        subject.instance_variable_get('@hour').should == 0
      end
    end

    context "when the time contains 12 pm" do
      it "should set hour as 12" do
        subject.send(:extract_hour, "12", "pm")
        subject.instance_variable_get('@hour').should == 12
      end
    end

    context "when the time is between 1 and 11 am" do
      it "should set hour between 1 to 11" do
        subject.send(:extract_hour, "9", "am")
        subject.instance_variable_get('@hour').should == 9
      end
    end

    context "when the time is between 1 and 12 pm" do
      it "should set hour between 12 to 23" do
        subject.send(:extract_hour, "3", "pm")
        subject.instance_variable_get('@hour').should == 15
      end
    end

    it "should return back the hour object" do
      subject.send(:extract_hour, "3", "pm").should eq(15)
    end
  end

  describe "#parse" do
    it "should parse the time and set the hours and mins" do
      subject.parse
      subject.instance_variable_get('@hour').should eq(19)
      subject.instance_variable_get('@min').should eq(45)
    end

    it "should return back the time parser object" do
      subject.parse.should eq(subject)
    end
  end
end