require 'spec_helper'

describe TimeKeeper::Parser::Military do
  subject do
    TimeKeeper::Parser::Military.new("19:45")
  end

  describe "#time_is_valid" do
    context "when hour and mins are both valid" do
      it "should return true" do
        subject.send(:time_is_valid).should be_true
      end
    end

    context "when either hour or min is not valid" do
      before(:each) do
        subject.instance_variable_set('@time_str', "19:60")
      end

      it "should return false" do
        subject.send(:time_is_valid).should be_false
      end

      it "should attach an error to the object" do
        subject.send(:time_is_valid)
        subject.errors[:base].should_not be_empty
      end
    end
  end

  describe "#extract_hour" do
    context "when hour is present" do
      it "should set hour" do
        subject.send(:extract_hour, "15")
        subject.instance_variable_get("@hour").should eq(15)
      end
    end

    context "when hour is not present" do
      it "should set hour to zero" do
        subject.send(:extract_hour, "")
        subject.instance_variable_get("@hour").should eq(0)
      end
    end

    it "should return back the hour object" do
      subject.send(:extract_hour, "3").should eq(3)
    end
  end

  describe "#parse" do
    it "should parse the time and set the hours and mins" do
      subject.parse
      subject.instance_variable_get('@hour').should eq(19)
      subject.instance_variable_get('@min').should eq(45)
    end

    it "should return back the time parser object" do
      subject.parse.should eq(subject)
    end
  end
end