require 'spec_helper'

describe TimeKeeper::TimeRange do
  subject do
    TimeKeeper::TimeRange.new("05:30 to 16:00")
  end

  describe "#initialize" do
    it "should set range" do
      TimeKeeper::TimeRange.new("05:30 am to 04:10 pm").instance_variable_get("@range").should eq("05:30 am-04:10 pm")
    end
  end

  describe "#parse" do
    it "should set the start time" do
      subject.parse
      subject.instance_variable_get("@start_time").in_minutes.should eq(330)
    end

    it "should set the start day" do
      subject.parse
      subject.instance_variable_get('@start_day').should eq(0)
    end

    it "should set the end time" do
      subject.parse
      subject.instance_variable_get('@end_time').in_minutes.should eq(960)
    end

    it "should set the end day" do
      subject.parse
      subject.instance_variable_get('@end_day').should eq(0)
    end

    it "should return the TimeKeeper::TimeRange object" do
      subject.parse.should eq(subject)
    end
  end

  describe "#days_elapsed" do
    context "when schedule passes to the next day" do
      before(:each) do
        subject.instance_variable_set("@start_time", TimeKeeper::CustomTime.new("22:00").parse)
        subject.instance_variable_set("@end_time", TimeKeeper::CustomTime.new("06:00").parse)
      end

      it "should set to next day" do
        subject.send(:days_elapsed).should eq(1)
      end
    end

    context "when schedule is on the same day" do
      before(:each) do
        subject.instance_variable_set("@start_time", TimeKeeper::CustomTime.new("05:30").parse)
        subject.instance_variable_set("@end_time", TimeKeeper::CustomTime.new("13:00").parse)
      end

      it "should set to next day to same day" do
        subject.send(:days_elapsed).should eq(0)
      end
    end
  end

  describe "#duration_in_seconds" do
    before(:each) do
      subject.range = "10:00 to 09:00"
      subject.parse
    end

    it "should return the duration of the time interval in seconds" do
      subject.duration_in_seconds.should eq(82800)
    end
  end

  describe "#duration_in_minutes" do
    before(:each) do
      subject.range = "10:00 to 09:00"
      subject.parse
    end

    it "should return the duration of the time interval in minutes" do
      subject.duration_in_minutes.should eq(1380)
    end
  end

  describe "#duration_in_hours" do
    before(:each) do
      subject.range = "10:00 to 09:00"
      subject.parse
    end

    it "should return the duration of the time interval in minutes" do
      subject.duration_in_hours.should eq(23)
    end
  end
end